/**********************************************************************
**
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
** Copyright (C) 2003 Germain Garand <germain@ebooksfrance.org>
**
** This file is part of Qt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "perleditor.h"
#include "syntaxhighliter_perl.h"
#include "perlindent.h"
#include "perlcompletion.h"
#include "perlbrowser.h"
#include "parenmatcher.h"
#include <qsettings.h>
#include <qpopupmenu.h>
#include <qinputdialog.h>
#include <qprocess.h>
#include "conf.h"
#include <designerinterface.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qevent.h>
#include "classbrowserinterfaceimpl.h"
#include "perlaction.h"

PerlEditor::PerlEditor( const QString &fn, QWidget *parent, const char *name, DesignerInterface *i, QComponentInformationInterface* ci )
    : Editor( fn, parent, name ), dIface( i ), cIface( ci )
{
    if ( dIface )
	dIface->addRef();
    if ( cIface )
        cIface->addRef();
    perl_process = 0;
    document()->setPreProcessor( new SyntaxHighlighter_Perl );
    document()->setIndent( (indent = new PerlIndent) );
    completion = new PerlEditorCompletion( this );
    browser = new PerlEditorBrowser( this );
    int j = 0;
    while ( SyntaxHighlighter_Perl::keywords[ j ] != QString::null )
	    completion->addCompletionEntry( SyntaxHighlighter_Perl::keywords[ j++ ], 0, FALSE );
    configChanged();
}

PerlEditor::~PerlEditor()
{
    delete completion;
    if ( dIface )
	dIface->release();
    if ( cIface )
        cIface->release();
    if ( perl_process )
    {
        if( perl_process->isRunning() )
	    perl_process->tryTerminate();
	delete perl_process;
    }
}

void PerlEditor::configChanged()
{
    QString path = "/Trolltech/PerlEditor/";
    QMap<QString, ConfigStyle> styles = Config::readStyles( path );
    config()->styles = styles;
    ( (SyntaxHighlighter_Perl*)document()->preProcessor() )->updateStyles( config()->styles );

    completion->setEnabled( Config::completion( path ) );
    parenMatcher->setEnabled( Config::parenMatching( path ) );
    if ( Config::wordWrap( path ) ) {
	if ( hScrollBarMode() != AlwaysOff ) {
	    document()->setFormatter( new QTextFormatterBreakInWords );
	    setHScrollBarMode( AlwaysOff );
	}
    } else {
	if ( hScrollBarMode() != AlwaysOn ) {
	    QTextFormatterBreakWords *f = new QTextFormatterBreakWords;
	    f->setWrapEnabled( FALSE );
	    document()->setFormatter( f );
	    setHScrollBarMode( AlwaysOn );
	}
    }

    setFont( ( (SyntaxHighlighter_Perl*)document()->preProcessor() )->format( 0 )->font() );

    indent->setTabSize( Config::indentTabSize( path ) );
    indent->setIndentSize( Config::indentIndentSize( path ) );
    indent->setKeepTabs( Config::indentKeepTabs( path ) );
    indent->setAutoIndent( Config::indentAutoIndent( path ) );
    if ( !Config::indentAutoIndent( path ) )
	document()->setIndent( 0 );
    else
	document()->setIndent( indent );

    document()->setTabStops( ( (SyntaxHighlighter_Perl*)document()->preProcessor() )->format( QTextPreProcessor::Standard )->width( 'x' ) * Config::indentTabSize( path ) );

    Editor::configChanged();
}

QPopupMenu *PerlEditor::createPopupMenu( const QPoint &p )
{
    QPopupMenu *m = Editor::createPopupMenu( p );
    m->insertSeparator();
    int addimplid = m->insertItem( tr( "Add Use Clause..." ), this, SLOT( addInclDecl() ) );
    if ( dIface->currentSourceFile() ) {
	m->setItemEnabled( addimplid, FALSE );
//	m->insertItem( tr( "Build all forms and run this file as main" )
//	m->setItemEnabled( addrunid, FALSE );
    }
    else
    {
     //   m->insertItem( tr( "Build and Run this form with perl" ), this, SLOT( buildAndRun() ) );
         QAction *a = new PerlAction( this, dIface, PerlAction::RunForm );
	 a->addTo( m );
    }
    return m;
}

void PerlEditor::addInclImpl()
{
}

void PerlEditor::addInclDecl()
{
    if ( !dIface )
	return;
    QString s = QInputDialog::getText( tr( "Add Use Clause" ),
				       tr( "e.g: <b>use Module;</b> or <b>use Qt::attributes qw( myVariable );</b>" ) );
    if ( s.isEmpty() )
	return;
    DesignerFormWindow *form = dIface->currentForm();
    QStringList lst = form->declarationIncludes();
    lst << s;
    form->setDeclarationIncludes( lst );
    if( form->form() )
    {
        //- hack to trigger setModified( true ) on current formFile
        QResizeEvent *ev = new QResizeEvent(
		form->form()->size(),
		form->form()->size() );
        QApplication::postEvent( form->form(), ev );
    }
}

void PerlEditor::addForward()
{
}

void PerlEditor::paste()
{
    Editor::paste();
    emit intervalChanged();
}
